﻿using System;
using System.Windows.Forms;

namespace CShp_GeraSenhas
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void chkRequerCaixaBaixa_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRequerCaixaBaixa.Checked) chkPermiteCaixaBaixa.Checked = true;
        }
        private void chkRequerCaixaAlta_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRequerCaixaAlta.Checked) chkPermiteCaixaAlta.Checked = true;
        }
        private void chkRequerNumeros_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRequerNumeros.Checked) chkPermiteNumeros.Checked = true;
        }
        private void chkRequerEspeciais_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRequerEspeciais.Checked) chkPermiteEspeciais.Checked = true;
        }
        private void chkRequerSublinhado_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRequerSublinhado.Checked) chkPermiteSublinhado.Checked = true;
        }
        private void chkRequerEspaco_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRequerEspaco.Checked) chkPermiteEspaco.Checked = true;
        }
        private void chkRequerOutros_CheckedChanged(object sender, EventArgs e)
        {
            if (chkRequerOutros.Checked) chkPermiteOutros.Checked = true;
        }

        // Não permitido implica que não é obrigatorio
        private void chkPermiteCaixaBaixa_CheckedChanged(object sender, EventArgs e)
        {
            if (!chkPermiteCaixaBaixa.Checked) chkRequerCaixaBaixa.Checked = false;
        }
        private void chkPermiteCaixaAlta_CheckedChanged(object sender, EventArgs e)
        {
            if (!chkPermiteCaixaAlta.Checked) chkRequerCaixaAlta.Checked = false;
        }
        private void chkPermiteNumeros_CheckedChanged(object sender, EventArgs e)
        {
            if (!chkPermiteNumeros.Checked) chkRequerNumeros.Checked = false;
        }
        private void chkPermiteEspeciais_CheckedChanged(object sender, EventArgs e)
        {
            if (!chkPermiteEspeciais.Checked) chkRequerEspeciais.Checked = false;
        }
        private void chkPermiteSublinhado_CheckedChanged(object sender, EventArgs e)
        {
            if (!chkPermiteSublinhado.Checked) chkRequerSublinhado.Checked = false;
        }
        private void chkPermiteEspaco_CheckedChanged(object sender, EventArgs e)
        {
            if (!chkPermiteEspaco.Checked) chkRequerEspaco.Checked = false;
        }
        private void chkPermiteOutros_CheckedChanged(object sender, EventArgs e)
        {
            if (!chkPermiteOutros.Checked) chkRequerOutros.Checked = false;
        }

        private void btnGerar_Click(object sender, EventArgs e)
        {
            try
            {
                txtSenha.Text = GeraSenhaAleatoria();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            txtSenha.SelectAll();
            txtSenha.Focus();
        }

        // Gera a senha conforme as definições
        private string GeraSenhaAleatoria()
        {
            const string CAIXA_BAIXA = "abcdefghijklmnopqrstuvwxyz";
            const string CAIXA_ALTA = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
            const string NUMEROS = "0123456789";
            const string ESPECIAIS = @"~!@#$%^&*():;[]{}<>,.?/\|";
            string outros = txtOutros.Text;

            if (chkRequerOutros.Checked && (outros.Length < 1))
            {
                MessageBox.Show("Você não pode exigir caracteres de uma string em branco.",
                    "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                txtOutros.Focus();
                return txtSenha.Text;
            }

            // Faz uma lista de caracteres permitidos
            string permitido = "";
            if (chkPermiteCaixaBaixa.Checked) permitido += CAIXA_BAIXA;
            if (chkPermiteCaixaAlta.Checked) permitido += CAIXA_ALTA;
            if (chkPermiteNumeros.Checked) permitido += NUMEROS;
            if (chkPermiteEspeciais.Checked) permitido += ESPECIAIS;
            if (chkPermiteSublinhado.Checked) permitido += "_";
            if (chkPermiteEspaco.Checked) permitido += " ";
            if (chkPermiteOutros.Checked) permitido += outros;

            // Obtem o numero de caracteres .
            int caracteres_minimo = int.Parse(txtMinimo.Text);
            int caracteres_maximo = int.Parse(txtMaximo.Text);
            int numero_caracteres = Crypto.RandomInteger(caracteres_minimo, caracteres_maximo);

            // Satisfaz as definições
            string _senha = "";
            if (chkRequerCaixaBaixa.Checked &&
                (_senha.IndexOfAny(CAIXA_BAIXA.ToCharArray()) == -1))
                _senha += RandomChar(CAIXA_BAIXA);
            if (chkRequerCaixaAlta.Checked &&
                (_senha.IndexOfAny(CAIXA_ALTA.ToCharArray()) == -1))
                _senha += RandomChar(CAIXA_ALTA);
            if (chkRequerNumeros.Checked &&
                (_senha.IndexOfAny(NUMEROS.ToCharArray()) == -1))
                _senha += RandomChar(NUMEROS);
            if (chkRequerEspeciais.Checked &&
                (_senha.IndexOfAny(ESPECIAIS.ToCharArray()) == -1))
                _senha += RandomChar(ESPECIAIS);
            if (chkRequerSublinhado.Checked &&
                (_senha.IndexOfAny("_".ToCharArray()) == -1))
                _senha += "_";
            if (chkRequerEspaco.Checked &&
                (_senha.IndexOfAny(" ".ToCharArray()) == -1))
                _senha += " ";
            if (chkRequerOutros.Checked &&
                (_senha.IndexOfAny(outros.ToCharArray()) == -1))
                _senha += RandomChar(outros);

            // adiciona os caracteres restantes aleatorios
            while (_senha.Length < numero_caracteres)
                _senha += RandomChar(permitido);

            // mistura os caracteres requeridos 
            _senha = RandomizeString(_senha);

            return _senha;
        }

        // retorna um caractere aleatorio a partir de uma string
        private string RandomChar(string str)
        {
            return str.Substring(Crypto.RandomInteger(0, str.Length - 1), 1);
        }

        // Retorna uma permutação aleatoria de uma string
        private string RandomizeString(string str)
        {
            string resultado = "";
            while (str.Length > 0)
            {
                // Pega um numero aleatorio
                int i = Crypto.RandomInteger(0, str.Length - 1);
                resultado += str.Substring(i, 1);
                str = str.Remove(i, 1);
            }
            return resultado;
        }
    }
}
